@@ -19,11 +19,11 @@ use crate::{errors::NetworkError, message::*, stats, Cache, ConnReader, ConnWrit
 use std::{collections::HashMap, net::SocketAddr, sync::Arc, time::Duration};
 
 use parking_lot::Mutex;
-use snarkvm_objects::Storage;
+use snarkvm_dpc::Storage;
 use tokio::{
     io::{AsyncReadExt, AsyncWriteExt},
     net::{TcpListener, TcpStream},
-    sync::mpsc::{channel, error::TrySendError},
+    sync::mpsc::error::TrySendError,
     task,
 };
 
@@ -65,14 +65,14 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
     /// This method handles new inbound connection requests.
     pub async fn listen(&self) -> Result<(), NetworkError> {
         let listener = TcpListener::bind(&self.config.desired_address).await?;
-        let listener_address = listener.local_addr()?;
+        let own_listener_address = listener.local_addr()?;
 
-        self.set_local_address(listener_address);
+        self.set_local_address(own_listener_address);
         info!("Initializing listener for node ({:x})", self.id);
 
         let node_clone = self.clone();
         let listener_handle = task::spawn(async move {
-            info!("Listening for nodes at {}", listener_address);
+            info!("Listening for nodes at {}", own_listener_address);
 
             loop {
                 match listener.accept().await {
@@ -89,46 +89,14 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
                             // Wait a maximum timeout limit for a connection request.
                             let handshake_result = tokio::time::timeout(
                                 Duration::from_secs(crate::HANDSHAKE_PEER_TIMEOUT_SECS as u64),
-                                node.connection_request(listener_address, remote_address, stream),
+                                node.connection_request(own_listener_address, remote_address, stream),
                             )
                             .await;
 
                             match handshake_result {
-                                Ok(Ok((mut writer, mut reader, remote_listener))) => {
-                                    // Create a channel dedicated to sending messages to the connection.
-                                    let (sender, receiver) = channel(crate::OUTBOUND_CHANNEL_DEPTH);
-
-                                    // Listen for inbound messages.
-                                    let node_clone = node.clone();
-                                    let peer_reading_task = tokio::spawn(async move {
-                                        node_clone.listen_for_inbound_messages(&mut reader).await;
-                                    });
-
-                                    // Listen for outbound messages.
-                                    let node_clone = node.clone();
-                                    let peer_writing_task = tokio::spawn(async move {
-                                        node_clone.listen_for_outbound_messages(receiver, &mut writer).await;
-                                    });
-
-                                    // Save the channel under the provided remote address.
-                                    node.outbound.channels.write().insert(remote_listener, sender);
-
-                                    // Finally, mark the peer as connected.
-                                    node.peer_book.set_connected(remote_address, Some(remote_listener));
-
-                                    trace!("Connected to {} (listener: {})", remote_address, remote_listener);
-
+                                Ok(Ok(remote_listener)) => {
                                     // Immediately send a ping to provide the peer with our block height.
                                     node.send_ping(remote_listener);
-
-                                    if let Ok(ref peer) = node.peer_book.get_peer(remote_listener) {
-                                        peer.register_task(peer_reading_task, true);
-                                        peer.register_task(peer_writing_task, false);
-                                    } else {
-                                        // If the related peer is not found, it means it's already been dropped.
-                                        peer_reading_task.abort();
-                                        peer_writing_task.abort();
-                                    }
                                 }
                                 Ok(Err(e)) => {
                                     error!("Failed to accept a connection request: {}", e);
@@ -236,61 +204,61 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
             Payload::Transaction(transaction) => {
                 metrics::increment_counter!(stats::INBOUND_TRANSACTIONS);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_memory_pool_transaction(source, transaction)?;
+                if self.sync().is_some() {
+                    self.received_memory_pool_transaction(source, transaction)?;
                 }
             }
             Payload::Block(block) => {
                 metrics::increment_counter!(stats::INBOUND_BLOCKS);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_block(source, block, true)?;
+                if self.sync().is_some() {
+                    self.received_block(source, block, true)?;
                 }
             }
             Payload::SyncBlock(block) => {
                 metrics::increment_counter!(stats::INBOUND_SYNCBLOCKS);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_block(source, block, false)?;
+                if self.sync().is_some() {
+                    self.received_block(source, block, false)?;
 
                     // Update the peer and possibly finish the sync process.
                     if self.peer_book.got_sync_block(source) {
-                        sync.finished_syncing_blocks();
+                        self.finished_syncing_blocks();
                     }
                 }
             }
             Payload::GetBlocks(hashes) => {
                 metrics::increment_counter!(stats::INBOUND_GETBLOCKS);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_get_blocks(source, hashes)?;
+                if self.sync().is_some() {
+                    self.received_get_blocks(source, hashes)?;
                 }
             }
             Payload::GetMemoryPool => {
                 metrics::increment_counter!(stats::INBOUND_GETMEMORYPOOL);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_get_memory_pool(source);
+                if self.sync().is_some() {
+                    self.received_get_memory_pool(source);
                 }
             }
             Payload::MemoryPool(mempool) => {
                 metrics::increment_counter!(stats::INBOUND_MEMORYPOOL);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_memory_pool(mempool)?;
+                if self.sync().is_some() {
+                    self.received_memory_pool(mempool)?;
                 }
             }
             Payload::GetSync(getsync) => {
                 metrics::increment_counter!(stats::INBOUND_GETSYNC);
 
-                if let Some(ref sync) = self.sync() {
-                    sync.received_get_sync(source, getsync)?;
+                if self.sync().is_some() {
+                    self.received_get_sync(source, getsync)?;
                 }
             }
             Payload::Sync(sync) => {
                 metrics::increment_counter!(stats::INBOUND_SYNCS);
 
-                if let Some(ref sync_handler) = self.sync() {
+                if self.sync().is_some() {
                     if sync.is_empty() {
                         // An empty `Sync` is unexpected, as `GetSync` requests are only
                         // sent to peers that declare a greater block height.
@@ -298,7 +266,7 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
                         warn!("{} doesn't have sync blocks to share", source);
                     } else if self.peer_book.expecting_sync_blocks(source, sync.len()) {
                         trace!("Received {} sync block hashes from {}", sync.len(), source);
-                        sync_handler.received_sync(source, sync);
+                        self.received_sync(source, sync);
                     }
                 }
             }
@@ -335,10 +303,10 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
     ///
     pub async fn connection_request(
         &self,
-        listener_address: SocketAddr,
+        own_listener_address: SocketAddr,
         remote_address: SocketAddr,
         stream: TcpStream,
-    ) -> Result<(ConnWriter, ConnReader, SocketAddr), NetworkError> {
+    ) -> Result<SocketAddr, NetworkError> {
         self.peer_book.set_connecting(remote_address)?;
 
         let (mut reader, mut writer) = stream.into_split();
@@ -366,8 +334,12 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
         trace!("received e (XX handshake part 1/3) from {}", remote_address);
 
         // -> e, ee, s, es
-        let own_version =
-            Version::serialize(&Version::new(crate::PROTOCOL_VERSION, listener_address.port(), self.id)).unwrap();
+        let own_version = Version::serialize(&Version::new(
+            crate::PROTOCOL_VERSION,
+            own_listener_address.port(),
+            self.id,
+        ))
+        .unwrap();
         let len = noise.write_message(&own_version, &mut buffer)?;
         writer.write_all(&[len as u8]).await?;
         writer.write_all(&buffer[..len]).await?;
@@ -391,16 +363,14 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
             return Err(NetworkError::InvalidHandshake);
         }
 
-        metrics::increment_counter!(stats::HANDSHAKES_SUCCESSES_RESP);
-
         // the remote listening address
         let remote_listener = SocketAddr::from((remote_address.ip(), peer_version.listening_port));
 
-        let noise = Arc::new(Mutex::new(noise.into_transport_mode()?));
-        let reader = ConnReader::new(remote_listener, reader, buffer.clone(), Arc::clone(&noise));
-        let writer = ConnWriter::new(remote_listener, writer, buffer, noise);
+        self.set_connected(remote_address, remote_listener, noise, buffer, reader, writer)?;
 
-        Ok((writer, reader, remote_listener))
+        metrics::increment_counter!(stats::HANDSHAKES_SUCCESSES_RESP);
+
+        Ok(remote_listener)
     }
 
     #[inline]
