@@ -15,9 +15,10 @@
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
 use crate::*;
-use snarkvm_objects::Storage;
+use snarkvm_dpc::Storage;
 
 use chrono::{DateTime, Utc};
+use metrics::{register_counter, register_gauge};
 use once_cell::sync::OnceCell;
 use parking_lot::Mutex;
 use rand::{seq::SliceRandom, thread_rng, Rng};
@@ -32,6 +33,9 @@ use std::{
 };
 use tokio::{task, time::sleep};
 
+#[cfg(feature = "prometheus")]
+use metrics_exporter_prometheus::PrometheusBuilder;
+
 #[derive(PartialEq, Eq, Clone, Copy, Debug)]
 #[repr(u8)]
 pub enum State {
@@ -213,14 +217,14 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
         });
         self.register_task(state_tracking_task);
 
-        if let Some(ref sync) = self.sync() {
+        if self.sync().is_some() {
             let bootnodes = self.config.bootnodes();
 
-            let sync_clone = Arc::clone(sync);
-            let mempool_sync_interval = sync_clone.mempool_sync_interval();
+            let node_clone = self.clone();
+            let mempool_sync_interval = node_clone.expect_sync().mempool_sync_interval();
             let sync_mempool_task = task::spawn(async move {
                 loop {
-                    if !sync_clone.is_syncing_blocks() {
+                    if !node_clone.is_syncing_blocks() {
                         // TODO (howardwu): Add some random sync nodes beyond this approach
                         //  to ensure some diversity in mempool state that is fetched.
                         //  For now, this is acceptable because we propogate the mempool to
@@ -236,7 +240,7 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
                         // Step 1.
                         let mut sync_node = None;
                         for bootnode in bootnodes.iter() {
-                            if sync_clone.node().peer_book.is_connected(*bootnode) {
+                            if node_clone.peer_book.is_connected(*bootnode) {
                                 sync_node = Some(*bootnode);
                                 break;
                             }
@@ -245,10 +249,10 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
                         // Step 2.
                         if sync_node.is_none() {
                             // Select last seen node as block sync node.
-                            sync_node = sync_clone.node().peer_book.last_seen();
+                            sync_node = node_clone.peer_book.last_seen();
                         }
 
-                        sync_clone.update_memory_pool(sync_node);
+                        node_clone.update_memory_pool(sync_node);
                     }
 
                     sleep(mempool_sync_interval).await;
@@ -256,12 +260,12 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
             });
             self.register_task(sync_mempool_task);
 
-            let sync_clone = Arc::clone(sync);
-            let block_sync_interval = sync_clone.block_sync_interval();
+            let node_clone = self.clone();
+            let block_sync_interval = node_clone.expect_sync().block_sync_interval();
             let sync_block_task = task::spawn(async move {
                 loop {
-                    let is_syncing_blocks = sync_clone.is_syncing_blocks();
-                    let is_sync_expired = sync_clone.has_block_sync_expired();
+                    let is_syncing_blocks = node_clone.is_syncing_blocks();
+                    let is_sync_expired = node_clone.expect_sync().has_block_sync_expired();
 
                     // if the node is not currently syncing blocks or an earlier sync attempt has expired,
                     // consider syncing blocks with a peer who has a longer chain
@@ -271,15 +275,15 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
                         // to deliver the batch of sync blocks
                         if is_syncing_blocks {
                             debug!("An unfinished block sync has expired.");
-                            sync_clone.node().set_state(State::Idle);
+                            node_clone.set_state(State::Idle);
                         }
 
                         let mut prospect_sync_nodes = Vec::new();
-                        let my_height = sync_clone.current_block_height();
+                        let my_height = node_clone.expect_sync().current_block_height();
 
                         // Pick a random peer of all the connected ones that claim
                         // to have a longer chain.
-                        for (peer, info) in sync_clone.node().peer_book.connected_peers().iter() {
+                        for (peer, info) in node_clone.peer_book.connected_peers().iter() {
                             // Fetch the current block height of this connected peer.
                             let peer_block_height = info.block_height();
 
@@ -300,11 +304,11 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
                             );
 
                             // Cancel any possibly ongoing sync attempts.
-                            sync_clone.node().peer_book.cancel_any_unfinished_syncing();
+                            node_clone.peer_book.cancel_any_unfinished_syncing();
 
                             // Begin a new sync attempt.
-                            sync_clone.register_block_sync_attempt();
-                            sync_clone.update_blocks(*sync_node);
+                            node_clone.register_block_sync_attempt();
+                            node_clone.update_blocks(*sync_node);
                         }
                     }
 
@@ -356,4 +360,77 @@ impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
             .set(addr)
             .expect("local address was set more than once!");
     }
+
+    #[cfg(feature = "prometheus")]
+    pub fn initialize_metrics(&self) {
+        debug!("Initializing prometheus metrics");
+
+        let prometheus_builder = PrometheusBuilder::new();
+
+        let (recorder, exporter) = prometheus_builder
+            .build_with_exporter()
+            .expect("can't build the prometheus exporter");
+        metrics::set_boxed_recorder(Box::new(recorder)).expect("can't set the prometheus exporter");
+
+        let metrics_exporter_task = task::spawn(async move {
+            exporter.await.expect("can't await the prometheus exporter");
+        });
+        self.register_task(metrics_exporter_task);
+    }
+
+    #[cfg(not(feature = "prometheus"))]
+    pub fn initialize_metrics(&self) {
+        debug!("Initializing RPC metrics");
+
+        metrics::set_recorder(&NODE_STATS).expect("couldn't initialize the metrics recorder!");
+    }
+
+    pub fn register_metrics(&self) {
+        register_counter!(crate::INBOUND_ALL_SUCCESSES);
+        register_counter!(crate::INBOUND_ALL_FAILURES);
+        register_counter!(crate::INBOUND_BLOCKS);
+        register_counter!(crate::INBOUND_GETBLOCKS);
+        register_counter!(crate::INBOUND_GETMEMORYPOOL);
+        register_counter!(crate::INBOUND_GETPEERS);
+        register_counter!(crate::INBOUND_GETSYNC);
+        register_counter!(crate::INBOUND_MEMORYPOOL);
+        register_counter!(crate::INBOUND_PEERS);
+        register_counter!(crate::INBOUND_PINGS);
+        register_counter!(crate::INBOUND_PONGS);
+        register_counter!(crate::INBOUND_SYNCS);
+        register_counter!(crate::INBOUND_SYNCBLOCKS);
+        register_counter!(crate::INBOUND_TRANSACTIONS);
+        register_counter!(crate::INBOUND_UNKNOWN);
+
+        register_counter!(crate::OUTBOUND_ALL_SUCCESSES);
+        register_counter!(crate::OUTBOUND_ALL_FAILURES);
+
+        register_counter!(crate::CONNECTIONS_ALL_ACCEPTED);
+        register_counter!(crate::CONNECTIONS_ALL_INITIATED);
+        register_counter!(crate::CONNECTIONS_ALL_REJECTED);
+        register_gauge!(crate::CONNECTIONS_CONNECTING);
+        register_gauge!(crate::CONNECTIONS_CONNECTED);
+        register_gauge!(crate::CONNECTIONS_DISCONNECTED);
+
+        register_counter!(crate::HANDSHAKES_FAILURES_INIT);
+        register_counter!(crate::HANDSHAKES_FAILURES_RESP);
+        register_counter!(crate::HANDSHAKES_SUCCESSES_INIT);
+        register_counter!(crate::HANDSHAKES_SUCCESSES_RESP);
+        register_counter!(crate::HANDSHAKES_TIMEOUTS_INIT);
+        register_counter!(crate::HANDSHAKES_TIMEOUTS_RESP);
+
+        register_gauge!(crate::QUEUES_INBOUND);
+        register_gauge!(crate::QUEUES_OUTBOUND);
+
+        register_counter!(crate::MISC_BLOCK_HEIGHT);
+        register_counter!(crate::MISC_BLOCKS_MINED);
+        register_counter!(crate::MISC_DUPLICATE_BLOCKS);
+        register_counter!(crate::MISC_DUPLICATE_SYNC_BLOCKS);
+        register_counter!(crate::MISC_RPC_REQUESTS);
+
+        // The node can already be at some non-zero height.
+        if let Some(sync) = self.sync() {
+            metrics::counter!(crate::MISC_BLOCK_HEIGHT, sync.current_block_height() as u64);
+        }
+    }
 }
