@@ -16,8 +16,7 @@
 
 use crate::{stats, Node, State};
 use snarkos_consensus::Miner;
-use snarkvm_dpc::{base_dpc::instantiated::*, AccountAddress};
-use snarkvm_objects::Storage;
+use snarkvm_dpc::{testnet1::instantiated::*, AccountAddress, Storage};
 
 use tracing::*;
 
@@ -110,7 +109,7 @@ impl<S: Storage + Send + Sync + 'static> MinerInstance<S> {
                     continue;
                 };
 
-                self.node.expect_sync().propagate_block(serialized_block, local_address);
+                self.node.propagate_block(serialized_block, local_address);
             }
         });
 
