@@ -17,11 +17,13 @@
 use crate::{Node, State};
 use snarkos_consensus::{ConsensusParameters, MemoryPool, MerkleTreeLedger};
 use snarkos_storage::BlockHeight;
-use snarkvm_dpc::base_dpc::{
-    instantiated::{Components, Tx},
-    parameters::PublicParameters,
+use snarkvm_dpc::{
+    testnet1::{
+        instantiated::{Components, Tx},
+        parameters::PublicParameters,
+    },
+    Storage,
 };
-use snarkvm_objects::Storage;
 
 use parking_lot::{Mutex, RwLock};
 use std::{
@@ -31,8 +33,6 @@ use std::{
 
 /// The sync handler of this node.
 pub struct Sync<S: Storage> {
-    /// The node this sync handler is bound to.
-    node: Node<S>,
     /// The core sync objects.
     pub consensus: Arc<snarkos_consensus::Consensus<S>>,
     /// If `true`, initializes a mining task on this node.
@@ -48,14 +48,12 @@ pub struct Sync<S: Storage> {
 impl<S: Storage> Sync<S> {
     #[allow(clippy::too_many_arguments)]
     pub fn new(
-        node: Node<S>,
         consensus: Arc<snarkos_consensus::Consensus<S>>,
         is_miner: bool,
         block_sync_interval: Duration,
         mempool_sync_interval: Duration,
     ) -> Self {
         Self {
-            node,
             consensus,
             is_miner,
             block_sync_interval,
@@ -64,11 +62,6 @@ impl<S: Storage> Sync<S> {
         }
     }
 
-    #[inline]
-    pub fn node(&self) -> &Node<S> {
-        &self.node
-    }
-
     /// Returns a reference to the storage system of this node.
     #[inline]
     pub fn storage(&self) -> &MerkleTreeLedger<S> {
@@ -99,16 +92,6 @@ impl<S: Storage> Sync<S> {
         self.is_miner
     }
 
-    /// Checks whether the node is currently syncing blocks.
-    pub fn is_syncing_blocks(&self) -> bool {
-        self.node.state() == State::Syncing
-    }
-
-    /// Register that the node is no longer syncing blocks.
-    pub fn finished_syncing_blocks(&self) {
-        self.node.set_state(State::Idle);
-    }
-
     /// Returns the current block height of the ledger from storage.
     #[inline]
     pub fn current_block_height(&self) -> BlockHeight {
@@ -125,12 +108,6 @@ impl<S: Storage> Sync<S> {
         }
     }
 
-    /// Register that the node attempted to sync blocks.
-    pub fn register_block_sync_attempt(&self) {
-        *self.last_block_sync.write() = Some(Instant::now());
-        self.node.set_state(State::Syncing);
-    }
-
     /// Returns the interval between each block sync.
     pub fn block_sync_interval(&self) -> Duration {
         self.block_sync_interval
@@ -145,3 +122,23 @@ impl<S: Storage> Sync<S> {
         self.consensus.parameters.max_block_size
     }
 }
+
+impl<S: Storage + Send + core::marker::Sync + 'static> Node<S> {
+    /// Checks whether the node is currently syncing blocks.
+    pub fn is_syncing_blocks(&self) -> bool {
+        self.state() == State::Syncing
+    }
+
+    /// Register that the node is no longer syncing blocks.
+    pub fn finished_syncing_blocks(&self) {
+        self.set_state(State::Idle);
+    }
+
+    /// Register that the node attempted to sync blocks.
+    pub fn register_block_sync_attempt(&self) {
+        if let Some(sync) = self.sync() {
+            *sync.last_block_sync.write() = Some(Instant::now());
+        }
+        self.set_state(State::Syncing);
+    }
+}
